/*
 *    NzdlServiceClient.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

//import java.io.*;
//import java.util.*;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.HashSet;
import java.util.Properties;

/* jdk 1.3 orb */
import org.omg.CORBA.ORB;
/* gsdl corba stuff */
import org.nzdl.gsdl.corba.gsdlInterface.*;
import org.nzdl.gsdl.util.*;

/**
 * Used to implement the {@link NzdlService NzdlService} object.
 * NzdlService is an object that services CORBA client requests to a
 * Greenstone Digital Library.
 *
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @author Aziz Mahoui    (amahoui@yahoo.com)
 * @author Gordon Paynter (paynter@cs.waikato.ac.nz)
 * @author Brett Sheeran (bsheeran@xtra.co.nz) (comments)
 * @version $Revision: 2339 $
 */
public class NzdlServiceClient 
  extends java.lang.Object 
  implements NzdlService {

  private corbaiface m_nzdlServer = null;

  /**
   * Creates an instance of Java's ORB (Object Request Broker) to interface
   * to a Greenstone Digital Library.
   * @param <b>args</b> command-line arguments from the application's main method, or
   * alternatively, the name of the applet. This parameter is passed to
   * <TT>ORB.init( _args, _props)</TT>. Args may be a <TT>null</TT>.
   * @param <b>props</b> application specific properties.
   * This parameter is passed to
   * <TT>ORB.init( _args, _props)</TT>. props may be a <TT>null</TT>.
   * @param <b>IOR</b> the Interoperable Object Reference, a string that identifies
   * the computer on which corbaserver is running and the interface it
   * supports.
   * @see The Java documentation on the ORB class.
   */
  public NzdlServiceClient( String [] _args, Properties _props, String _IOR) {
    corbaComErrorHolder error = NzdlCorbaFactory.createComErrorHolder();

    try {

      // getting a reference to the remote NZDL server
      org.omg.CORBA.ORB orb = ORB.init( _args, _props);
      org.omg.CORBA.Object obj = orb.string_to_object(_IOR) ;
      m_nzdlServer = corbaifaceHelper.narrow(obj);
      
      // basic setup
      if (m_nzdlServer.initialise(error)) {
      } else {
	  throw new Error("NzdlServiceClient initialisation unsuccessful.");
      }
      
    } catch (Exception e) {
      System.err.println("Exception in NzdlServiceClient initialisation: \n" + e) ;
      System.err.println("corbaComError value: " + error.value.value());
      System.err.println("NzdlServiceClient initialisation unsuccessful.");
      throw new Error("NzdlServiceClient initialisation unsuccessful.");
    }
    if (error.value.value() != corbaComError._corbaNoError) {
      System.err.println("corbaComError value: " + error.value.value());
    }
    
    System.out.println("NzdlServiceClient initialisation successful.");
  }


  /**
   * Allow the client to configure a CORBA server.
   * @param key the parameter to be set.
   * @param values what that parameter will be set to.
   */
  public void configure( String _key, Set _values ) {
    if (_key == null || _key.equals("")){
      throw new Error ("null or empty collection _key");
    }
    if (_values == null ){
      throw new Error ("null or empty collection _values");
    }
    corbaComErrorHolder error = NzdlCorbaFactory.createComErrorHolder();
    m_nzdlServer.configure( NzdlCorbaFactory.toCorbaText( _key ),
			    NzdlCorbaFactory.toCorbaTextArray( _values ),
			    error);
    if (error.value.value() != corbaComError._corbaNoError) {
      System.err.println("CORBA error:" + error.value.value());
    }
  }

  /**
   * Obtain the set of collection names for a Greenstone library.
   * @return A set of collection names in string format.
   */
  public Set getCollectionSet( ) {
    System.err.println("Client::getCollectionSet() start");
    corbaComErrorHolder error = NzdlCorbaFactory.createComErrorHolder();
    corbatext_tarrayHolder c_collSet = NzdlCorbaFactory.createCorbaTextArrayHolder();
    m_nzdlServer.collectionList(c_collSet, error);
    Set collSet = new HashSet();
    for (int i=0; i<c_collSet.value.length; i++) {
      String collName = NzdlCorbaFactory.toString( c_collSet.value[i] );
      if (collName.indexOf("nuipepa") == -1)
	collSet.add(collName);
    }
    if (error.value.value() != corbaComError._corbaNoError) {
      System.err.println("CORBA error:" + error.value.value());
    }
    System.err.println("Client::getCollectionSet() end " + error.value.value());
    return collSet ;
  }

  /**
   * Check if ORB knows of this collection.
   * @param name the name of a collecion to be searched for.
   * @return boolean <TT>true</TT> if collection is found, else <TT>false</TT>.
   */
  public boolean hasCollection( String _name ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty collection _name");
    }
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    org.omg.CORBA.BooleanHolder c_has = new org.omg.CORBA.BooleanHolder();
    m_nzdlServer.hasCollection( NzdlCorbaFactory.toCorbaText(_name), c_has, c_err );
    if (c_err.value.value() != 0)
      System.err.println("hasCollection error " + c_err.value.value());
    return c_has.value;
  }


  /**
   * Check if ORB can communicate with of this collection.
   * @param name the name of a collecion to be pinged.
   * @return boolean <TT>true</TT> if collection returned ping, else
   * <TT>false</TT>.
   */
  public boolean pingCollection( String _name ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty collection _name");
    }
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    org.omg.CORBA.BooleanHolder c_ping = new org.omg.CORBA.BooleanHolder();
    m_nzdlServer.ping( NzdlCorbaFactory.toCorbaText(_name), c_ping, c_err );
    if (c_err.value.value() != 0)
      System.err.println("pingCollection() error " + c_err.value.value());
    return c_ping.value;
  }

  /**
   * Extract information on this collection into a
   * {@link NzdlCollectionInfo NzdlCollectionInfo} object
   * @param name the collection to be retrieved.
   * @return an instance of
   * {@link NzdlCollectionInfo NzdlCollectionInfo} containing data on a
   * collection
   */
  public NzdlCollectionInfo getCollectionInfo( String _name ) {
    System.err.println("getCollectionInfo " + _name);
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty collection _name");
    }
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    corbaColInfoResponseHolder c_info = NzdlCorbaFactory.createColInfoResponseHolder();
    m_nzdlServer.getCollectInfo( NzdlCorbaFactory.toCorbaText(_name), c_info, c_err );
    if (c_err.value.value() != 0)
      System.err.println("getCollectionInfo() error " + c_err.value.value());
    return new NzdlCollectionInfo( c_info.value );
  }

  /**
   * Returns the set of filters for the service object.
   * @param name the collection name
   * @return a set of strings that name the filters, for example
   * BrowseFilter, QueryFilter and NullFilter.
   */
  public Set getFilterSet( String _name ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty filter _name");
    }
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    corbatext_tarrayHolder c_collSet = NzdlCorbaFactory.createCorbaTextArrayHolder();
    m_nzdlServer.getFilterInfo(c_name, c_collSet,c_err);
    if (c_err.value.value() != 0)
      System.err.println("getCollectionInfo() error " + c_err.value.value());

    Set collSet = new HashSet();
    for (int i=0; i<c_collSet.value.length; i++) {
      collSet.add( NzdlCorbaFactory.toString( c_collSet.value[i] ) );
    }
    return collSet ;

  }

  /**
   * Not Currently implemented.
   */
  public Set getMetaTagSet( String _name ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty _name");
    }
    System.err.println("TO BE IMPLEMENTED: DON'T KNOW HOW YET !!! (getMetaTagSet)");
    return null ;
  }

  /**
   * Returns the Greenstone Markup Language (GML) text for a particular
   * document from a collection.
   * @param name the name of the collection.
   * @param docID the document identity string
   * @return the GML document in the form of a string.
   */
  public String getDocument( String _name, String _docID ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty _name");
    }
    if (_docID == null || _docID.equals("")){
      throw new Error ("null or empty _docID");
    }
    corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    corbaDocRequestHolder c_request = NzdlCorbaFactory.createDocRequestHolder( _docID );
    corbaDocResponseHolder c_response = NzdlCorbaFactory.createDocResponseHolder( );
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    m_nzdlServer.getDocument( c_name, c_request, c_response, c_err );
    if (c_err.value.value() != 0)
      System.err.println("getDocument() error " + c_err.value.value());
    return NzdlCorbaFactory.toString( c_response.value.doc );
  }

  /**
   * Returns the set of value(s) for a metatag from a particular document.
   * @param name collection name
   * @param docID the document identifier.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The set of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Set getMetaData( String _name, String _docID, String _metaTag ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty _name");
    }
    if (_docID == null || _docID.equals("")){
      throw new Error ("null or empty _docID");
    }
    if (_metaTag == null || _metaTag.equals("")){
      throw new Error ("null or empty _metaTag");
    }
    NzdlRequest request  = new NzdlRequest( _docID, _metaTag );
    NzdlResponse response = new NzdlResponse( );
    service( _name, request, response );
    if (response == null || response.getResponse() == null) {
      return new HashSet();
    }
    NzdlResultSet resultSet = response.getResultSet();
    /*
     * Because a result set may contain several hits, we only return
     * the set of metadata values of the specifed meta tag for the
     * specified document ID.
     */
    return resultSet.getMetaData( _docID, _metaTag );
  }

  /**
   * Returns a map of value(s) for a metatag from a list of documents.
   * @param name collection name
   * @param docIDs a list of document identifier strings.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The map of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Map getMetaData( String _name, List _docIDs, String _metaTag ) {
    if (_name == null || _name.equals("")){
      throw new Error ("null or empty _name");
    }
    if (_docIDs == null){
      throw new Error ("null _docIDs");
    }
    if (_metaTag == null || _metaTag.equals("")){
      throw new Error ("null or empty _metaTag");
    }
    NzdlRequest request  = new NzdlRequest( _docIDs, _metaTag );
    NzdlResponse response = new NzdlResponse( );
    service( _name, request, response );
    NzdlResultSet resultSet = response.getResultSet();
    //return resultSet.getMetaData( _docID, _metaTag );
    return resultSet.getMetaData( _metaTag );
    //return resultSet.getAllMetaData( );
  }

  /**
   * Services a {@link NzdlRequest NzdlRequest} to a collection and places
   * the results into a {@link NzdlResponse NzdlResponse} object. <br>
   * Pre:   Created instances of NzdlRequest and NzdlResponse. <br>
   * Post: Placed the result data into NzdlResponse object.
   * @param name the collection name.
   * @param request an instance of NzdlRequest with query data to be serviced
   * @param response an instance of NzdlResponse. where the results of
   * sevciving the query will be placed
   */
  public void service( String _name, NzdlRequest _request, NzdlResponse _response ) {    
    if (_name == null || _name.equals("")) {
      throw new Error ("null or empty _name");
    }
    if (_request == null) {
      throw new Error ("null _request");
    }
    if (_response == null) {
      throw new Error ("null _response");
    }
    corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    m_nzdlServer.filter( c_name, _request.getFilter(), _response.getHolder(), c_err );
    if (c_err.value.value() != 0) {
      System.err.println("service() error " + c_err.value.value());
      new Error().printStackTrace();
    }
  }
  
}
